package class7_2_TestsBroken;

import class7_2_TestsFixed.ConcreteTest;
import class7_2_TestsFixed.LinkedList;

import java.util.Iterator;
import java.util.List;

public class TestDriverLinkedList {
    public static void main(String[] args) {
        new TestDriverLinkedList().test();
    }

    private void test() {
        TestSet tests = new TestSet();
        tests.addTest(new ConcreteTest("LinkedList: Simple add(E)", new Runnable() {
            @Override
            public void run() {
                simpleAdd();
            }
        }));
        tests.addTest(new ConcreteTest("LinkedList: repeated add(E)", new Runnable() {
            @Override
            public void run() {
                repeatedAdd();
            }
        }));
        tests.addTest(new ConcreteTest("LinkedList: repeated add(0,E)", new Runnable() {
            @Override
            public void run() {
                addAtFront();
            }
        }));
        tests.addTest(new ConcreteTest("LinkedList: add at end with add(int,E)",
                new Runnable() {
            @Override
            public void run() {
                addAtEndIndexed();
            }
        }));
        tests.addTest(new ConcreteTest("LinkedList: iterate list explicitly",
                new Runnable() {
            @Override
            public void run() {
                simpleExplicitIterator();}
        }));
        tests.addTest(new ConcreteTest("LinkedList: iterate list implicitly",
                new Runnable() {
                    @Override
                    public void run() {
                        simpleImplicitIterator();}
                }));
        tests.addTest(new ConcreteTest("LinkedList: check values with iterator",
                new Runnable() {
                    @Override
                    public void run() {
                        testIterator();}
                }));

        tests.test();
    }

    private void simpleAdd() {
        List<String> list = new LinkedList<String>();
        list.add("Fun!");
    }

    private void repeatedAdd() {
        List<String> list = new LinkedList<String>();
        list.add("Fun!");
        list.add("More Fun!");
        list.add("Even more fun!");
        if(!list.get(0).equals("Fun!")) {
            throw new RuntimeException("Wrong first element.");
        }
        if(!list.get(1).equals("More Fun!")) {
            throw new RuntimeException("Wrong second element.");
        }
        if(!list.get(2).equals("Even more fun!")) {
            throw new RuntimeException("Wrong third element.");
        }
    }

    private void checkElement(List<String> list, int index, String expected) {
        if(!list.get(index).equals(expected)) {
            throw new RuntimeException("Wrong value at index "+index+" expected: "+expected+" got: "+list.get(index));
        }
    }

    private void addAtFront() {
        List<String> list = new LinkedList<String>();
        list.add(0,"test");
        list.add(0,"a");
        list.add(0,"is");
        list.add(0,"this");
        checkElement(list, 0, "this");
        checkElement(list, 1, "is");
        checkElement(list, 2, "a");
        checkElement(list, 3, "test");
    }

    private void addAtEndIndexed() {
        List<String> list = new LinkedList<String>();
        list.add(list.size(),"this");
        list.add(list.size(),"is");
        list.add(list.size(),"a");
        list.add(list.size(),"test");
        checkElement(list, 0, "this");
        checkElement(list, 1, "is");
        checkElement(list, 2, "a");
        checkElement(list, 3,"test");
    }


    private void simpleExplicitIterator() {
        List<String> list = new LinkedList<String>();
        list.add(list.size(), "this");
        list.add(list.size(), "is");
        list.add(list.size(), "a");
        list.add(list.size(), "test");

        Iterator<String> iterator = list.iterator();
        while(iterator.hasNext()) {
            String token = iterator.next();
            System.out.print(token + " ");
        }
        System.out.println();
    }

    private void simpleImplicitIterator() {
        List<String> list = new LinkedList<String>();
        list.add(list.size(), "this");
        list.add(list.size(), "is");
        list.add(list.size(), "a");
        list.add(list.size(), "test");

        for (String token : list) {
            System.out.print(token + " ");
        }
        System.out.println();
    }

    private void testIterator() {
        List<String> actualList = new LinkedList<String>();
        actualList.add(actualList.size(), "this");
        actualList.add(actualList.size(), "is");
        actualList.add(actualList.size(), "a");
        actualList.add(actualList.size(), "test");

        String[] expectedArray = {"this", "is", "a", "test"};

        Iterator<String> iterator = actualList.iterator();
        for(int i=0; i<expectedArray.length; i++) {
            if(!iterator.hasNext()) {
                throw new RuntimeException("Iterator should have a \"next\" at index "+i);
            }
            String token = iterator.next();
            if(!token.equals(expectedArray[i])) {
                throw new RuntimeException("Iterator should have \""+expectedArray+"\", got "+token);
            }
        }
        System.out.println();
    }
}
