package class7_2_TestsFixed;

public class TestDriverCircularQueue {
    public static void main(String[] args) {
        new TestDriverCircularQueue().test();
    }

    private void test() {
        TestSet tests = new TestSet();
        tests.addTest(new ConcreteTest("CircularQueue: Simple offer(E)",
                new Runnable() {
            @Override
            public void run() {
                simpleEnqueue();
            }
        }));
        tests.addTest(new ConcreteTest("CircularQueue: repeated offer(0,E)",
                new Runnable() {
            @Override
            public void run() {
                repeatedOffer();
            }
        }));
        tests.addTest(new ConcreteTest("CircularQueue: improved repeated offer(0,E)",
                new Runnable() {
            @Override
            public void run() {
                improvedOfferTest();
            }
        }));
        tests.addTest(new ConcreteTest("CircularQueue: add at end with add(int,E)",
                new Runnable() {
            @Override
            public void run() {
                checkFillingQueue();
            }
        }));
        tests.addTest(new ConcreteTest("CircularQueue: short emptying test",
                new Runnable() {
            @Override
            public void run() {
                shortCheckEmptyingQueue();
            }
        }));
        tests.addTest(new ConcreteTest("CircularQueue: full emptying test",
                new Runnable() {
                    @Override
                    public void run() {
                        checkEmptyingFullQueue();
                    }
                }));
        tests.addTest(new ConcreteTest("CircularQueue: over-full emptying test",
                new Runnable() {
                    @Override
                    public void run() {
                        checkEmptyingOverfullQueue();
                    }
                }));
        tests.addTest(new ConcreteTest("CircularQueue: emptying test (last element at 0)",
                new Runnable() {
                    @Override
                    public void run() {
                        checkEmptyingQueueLastElementAt0();
                    }
                }));
        tests.addTest(new ConcreteTest("CircularQueue: emptying test (last element at 2)",
                new Runnable() {
                    @Override
                    public void run() {
                        checkEmptyingQueueLastElementAt2();
                    }
                }));
        tests.addTest(new ConcreteTest("CircularQueue: simple peek",
                new Runnable() {
                    @Override
                    public void run() {
                        checkSimplePeek();
                    }
                }));
        tests.addTest(new ConcreteTest("CircularQueue: empty peek",
                new Runnable() {
                    @Override
                    public void run() {
                        checkEmptyPeek();
                    }
                }));
        tests.addTest(new ConcreteTest("CircularQueue: overfull index 1 peek",
                new Runnable() {
                    @Override
                    public void run() {
                        checkOverfullIndex1Peek();
                    }
                }));

        tests.test();
    }

    private void simpleEnqueue() {
        Queue<String> queue = new CircularQueue<String>();
        queue.offer("Fun!");
    }

    private void repeatedOffer() {
        Queue<String> queue = new CircularQueue<String>();
        queue.offer("Fun!");
        queue.offer("More Fun!");
        queue.offer("Even more fun!");
        if(!queue.poll().equals("Fun!")) {
            throw new RuntimeException("Wrong first element.");
        }
        if(!queue.poll().equals("More Fun!")) {
            throw new RuntimeException("Wrong second element.");
        }
        if(!queue.poll().equals("Even more fun!")) {
            throw new RuntimeException("Wrong third element.");
        }
    }

    private void checkPoll(Queue<String> queue, String expected) {
        String actual = queue.poll();
        if(actual == null && actual != expected || actual !=null && !actual.equals(expected)) {
            throw new RuntimeException("Wrong value when polling.  Expected: "+expected+" got: "+actual);
        }
    }

    private void checkPeek(Queue<String> queue, String expected) {
        String actual = queue.peek();
        if(actual == null && actual != expected || actual !=null && !actual.equals(expected)) {
            throw new RuntimeException("Wrong value when polling.  Expected: "+expected+" got: "+actual);
        }
    }

    private void improvedOfferTest() {
        Queue<String> queue = new CircularQueue<String>();
        queue.offer("this");
        queue.offer("is");
        queue.offer("a");
        queue.offer("test");
        checkPoll(queue, "this");
        checkPoll(queue, "is");
        checkPoll(queue, "a");
        checkPoll(queue, "test");
    }

    private void checkOffer(Queue<String> queue, String toOffer, boolean expectedReturnValue) {
        boolean actualReturnValue = queue.offer(toOffer);
        if(actualReturnValue != expectedReturnValue) {
            throw new RuntimeException("Error: Unexpected return value while offering an item. " +
                    "expected: "+expectedReturnValue+" actual: "+actualReturnValue);
        }
    }

    private void checkFillingQueue() {
        Queue<String> queue = new CircularQueue<String>();
        checkOffer(queue, "this", true);
        checkOffer(queue, "is", true);
        checkOffer(queue, "a", true);
        checkOffer(queue, "long", true);
        checkOffer(queue, "test", true);
        checkOffer(queue, "and...", false);
    }

    private void shortCheckEmptyingQueue() {
        Queue<String> queue = new CircularQueue<String>();
        checkPoll(queue, null); // queue should be empty.
    }

    private void checkEmptyingFullQueue() {
        Queue<String> queue = new CircularQueue<String>();
        checkOffer(queue, "this", true);
        checkOffer(queue, "is", true);
        checkOffer(queue, "a", true);
        checkOffer(queue, "long", true);
        checkOffer(queue, "test", true);
        checkPoll(queue, "this");
        checkPoll(queue, "is");
        checkPoll(queue, "a");
        checkPoll(queue, "long");
        checkPoll(queue, "test");
        checkPoll(queue, null); // queue should be empty.
    }

    /**
     * This tests emptying a queue where the last element removed is at
     * the internal array.length-1 (last element in the array)
     */
    private void checkEmptyingOverfullQueue() {
        Queue<String> queue = new CircularQueue<String>();
        checkOffer(queue, "this", true);
        checkOffer(queue, "is", true);
        checkOffer(queue, "a", true);
        checkOffer(queue, "long", true);
        checkOffer(queue, "test", true);
        checkOffer(queue, "oops!", false);
        checkPoll(queue, "this");
        checkPoll(queue, "is");
        checkPoll(queue, "a");
        checkPoll(queue, "long");
        checkPoll(queue, "test");
        checkPoll(queue, null); // queue should be empty.
    }

    /**
     * This tests emptying a queue where the last element removed is at
     * the internal array[0] (first element in the array)
     */
    private void checkEmptyingQueueLastElementAt0() {
        Queue<String> queue = new CircularQueue<String>();

        // Fill queue
        checkOffer(queue, "this", true);
        checkOffer(queue, "is", true);
        checkOffer(queue, "a", true);
        checkOffer(queue, "long", true);
        checkOffer(queue, "test", true);

        // Empty spot index 0
        checkPoll(queue, "this");

        // Fill it again
        checkOffer(queue, "watermelon", true);

        // Over-fill, just for good measure.
        checkOffer(queue, "oops!", false);

        // Now empty the queue.
        checkPoll(queue, "is");
        checkPoll(queue, "a");
        checkPoll(queue, "long");
        checkPoll(queue, "test");
        checkPoll(queue, "watermelon");
        checkPoll(queue, null); // queue should be empty.
    }


    /**
     * This tests emptying a queue where the last element removed is at
     * the internal array[0] (first element in the array)
     */
    private void checkEmptyingQueueLastElementAt2() {
        Queue<String> queue = new CircularQueue<String>();

        // Fill queue
        checkOffer(queue, "this", true);
        checkOffer(queue, "is", true);
        checkOffer(queue, "a", true);
        checkOffer(queue, "long", true);
        checkOffer(queue, "test", true);

        // Empty spot index 0, 1, and 2
        checkPoll(queue, "this");
        checkPoll(queue, "is");
        checkPoll(queue, "a");

        // Fill it again
        checkOffer(queue, "i0", true);
        checkOffer(queue, "i1", true);
        checkOffer(queue, "i2", true);

        // Over-fill, just for good measure.
        checkOffer(queue, "oops!", false);

        // Now empty the queue.
        checkPoll(queue, "long");
        checkPoll(queue, "test");
        checkPoll(queue, "i0");
        checkPoll(queue, "i1");
        checkPoll(queue, "i2");
        checkPoll(queue, null); // queue should be empty.
    }

    private void checkSimplePeek() {
        Queue<String> queue = new CircularQueue<String>();
        checkOffer(queue, "Away", true);
        checkPeek(queue, "Away");
    }

    private void checkEmptyPeek() {
        Queue<String> queue = new CircularQueue<String>();
        checkPeek(queue, null);
    }

    private void checkOverfullIndex1Peek() {
        Queue<String> queue = new CircularQueue<String>();
        checkOffer(queue, "Away", true);
        checkOffer(queue, "in", true);
        checkOffer(queue, "a", true);
        checkOffer(queue, "manger", true);
        checkOffer(queue, "no", true);

        checkPoll(queue, "Away");
        checkPoll(queue, "in");

        checkOffer(queue, "room", true);
        checkOffer(queue, "for", true);
        checkOffer(queue, "a bed", false);

        checkPeek(queue, "a");
    }
}
